package com.company.rowlevelerror

import com.company.rowlevelerror.app.FixedJpaDataStore
import io.jmix.core.impl.JpaStoreDescriptor
import io.jmix.eclipselink.impl.JpaDataStore
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.config.BeanDefinition
import org.springframework.boot.SpringApplication
import org.springframework.boot.autoconfigure.SpringBootApplication
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties
import org.springframework.boot.context.event.ApplicationStartedEvent
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.DependsOn
import org.springframework.context.annotation.Primary
import org.springframework.context.annotation.Scope
import org.springframework.context.event.EventListener
import org.springframework.core.env.Environment
import org.springframework.stereotype.Component

@SpringBootApplication
open class RowLevelErrorApplication {

    companion object {
        @JvmStatic
        fun main(args: Array<String>) {
            SpringApplication.run(RowLevelErrorApplication::class.java, *args)
        }
    }

    @Autowired
    private val environment: Environment? = null

    @Bean
    @Primary
    @ConfigurationProperties("main.datasource")
    open fun dataSourceProperties(): DataSourceProperties = DataSourceProperties()


    @Bean("fix_JpaDataStore")
    @Primary
    @Scope(BeanDefinition.SCOPE_PROTOTYPE)
    open fun jpaDataStore():JpaDataStore = FixedJpaDataStore()

    @Bean("fix_JpaStoreDescriptor")
    @Primary
    open fun JpaStoreDescriptorr(): JpaStoreDescriptor {
        return object : JpaStoreDescriptor() {
            override fun getBeanName() = "fix_JpaDataStore"
        }
    }


    @EventListener
    open fun printApplicationUrl(event: ApplicationStartedEvent?) {
        LoggerFactory.getLogger(RowLevelErrorApplication::class.java).info(
                "Application started at http://localhost:"
                        + (environment?.getProperty("local.server.port") ?: "")
                        + (environment?.getProperty("server.servlet.context-path") ?: ""))
    }
}