package com.company.rowlevelerror.app

import io.jmix.core.LoadContext
import io.jmix.core.datastore.DataStoreBeforeEntityCountEvent
import io.jmix.core.datastore.DataStoreEntityLoadingEvent
import io.jmix.core.datastore.EventSharedState
import io.jmix.eclipselink.impl.JpaDataStore
import org.slf4j.LoggerFactory

class FixedJpaDataStore : JpaDataStore() {
    override fun getCount(context: LoadContext<*>): Long {
        if (log.isDebugEnabled) {
            log.debug("getCount: store={}, metaClass={}, from selected={}, query={}",
                    name, context.entityMetaClass,
                    context.previousQueries.isEmpty(), context.query)
        }
        val eventState = EventSharedState()
        val beforeCountEvent = DataStoreBeforeEntityCountEvent(context, eventState)
        fireEvent(beforeCountEvent)
        if (beforeCountEvent.countPrevented()) {
            return 0
        }
        var count = 0L
        val transaction = beginLoadTransaction(context.isJoinTransaction)
        try {
            count = if (beforeCountEvent.countByItems()) {
                val countContext = context.copy()
                if (countContext.query != null) {
                    countContext.query!!.setFirstResult(0)
                    countContext.query!!.setMaxResults(0)
                }
                val entities = loadAll(countContext) //fix List<?> -> List<Object>
                val loadEvent = DataStoreEntityLoadingEvent(context, entities, eventState)
                fireEvent(loadEvent)
                val resultList = loadEvent.resultEntities
                resultList.size.toLong()
            } else {
                countAll(context)
            }
            beforeLoadTransactionCommit(context, emptyList())
            commitTransaction(transaction)
        } finally {
            rollbackTransaction(transaction)
        }
        return count
    }

    companion object {
        private val log = LoggerFactory.getLogger(FixedJpaDataStore::class.java)
    }
}
